#pragma once
#include "domain/domain.h"
#include <boost/uuid/uuid.hpp>


namespace spin
{
    class IBridgeClient
    {
    public:
        virtual ~IBridgeClient() = default;

        virtual void setHost(const std::string &host) = 0;
        virtual std::string host() const = 0;
        virtual void connect() const = 0;

        virtual MonitoredItems monitoredItems() const = 0;
        virtual BridgeInfo bridgeInfo() const = 0;
        virtual ToolInfo toolInfo() const = 0;
        virtual void enableExhibitionMode() const = 0;
        virtual std::set<Program> programs() const = 0;
        virtual std::set<Group> groups() const = 0;
        virtual void acknowledgeErrors() const = 0;
        virtual void safetyShieldDoHoming() const = 0;
        virtual void safetyShieldStop() const = 0;
        virtual void safetyShieldLockBit() const = 0;
        virtual void safetyShieldReleaseBit() const = 0;
        virtual void safetyShieldCoverBit() const = 0;
        virtual void safetyShieldCoverScrew(const double &length_mm) const = 0;
        virtual void safetyShieldFollowRobot() const = 0;
        virtual void safetyShieldMoveTo(const double &length_mm) const = 0;
        virtual bool isSafetyShieldIdle() const = 0;
        virtual void startMotor(const int speed_rpm, const int rampTime_ms = 500) const = 0;
        virtual void stopMotor() const = 0;
        virtual ProgramResult latestResult() const = 0;
        virtual UUID latestResultUUID() const = 0;
        virtual ProgramResult::Evaluation latestResultEvaluation() const = 0;
        virtual std::string programMessages() const = 0;
        virtual double drivingSpeedSetpoint() const = 0;
        virtual UUID startScrewProgram(const UUID &programUuid) const = 0;
    };
} // namespace spin