#pragma once
#include <iostream>
#include <sstream>

namespace spin
{
    class Length
    {
    private:
        double length_m;

        Length(const double &length_m) : length_m(length_m) {}

    public:
        static const Length fromMeters(const double &length_m)
        {
            return Length(length_m);
        }

        static const Length fromMillimeters(const double &lengthMm)
        {
            return Length(lengthMm / 1000);
        }

        double toMeters() const
        {
            return length_m;
        }

        double toMillimeters() const
        {
            return length_m * 1000;
        }

        Length abs() const
        {
            return Length(std::abs(length_m));
        }

        Length operator+(const Length &rhs) const
        {
            return Length(length_m + rhs.length_m);
        }

        Length operator-(const Length &rhs) const
        {
            return Length(length_m - rhs.length_m);
        }

        Length operator-() const 
        {
            return Length(-length_m);
        }

        friend std::ostream &operator<<(std::ostream &out, const Length &length)
        {
            out << length.toMillimeters() << " mm";
            return out;
        }

        std::string toString() const
        {
            std::ostringstream oss;
            oss << *this;
            return oss.str();
        }
    };

    class Speed
    {
    private:
        double speed_m_s;

        Speed(const double &speed_m_s) : speed_m_s(speed_m_s) {}

    public:
        static const Speed fromMetersPerSecond(const double &speed_m_s)
        {
            return Speed(speed_m_s);
        }

        static const Speed fromMillimetersPerSecond(const double &speed_mm_s)
        {
            return Speed(speed_mm_s / 1000);
        }

        double toMetersPerSecond() const
        {
            return speed_m_s;
        }

        double toMillimetersPerSecond() const
        {
            return speed_m_s * 1000;
        }

        Speed operator-() const
        {
            return Speed(-speed_m_s);
        }

        friend std::ostream &operator<<(std::ostream &out, const Speed &speed)
        {
            out << speed.toMillimetersPerSecond() << " mm/s";
            return out;
        }
    };
} // namespace spin